@@ -9,7 +9,7 @@ Digest only the first or only the second tatum of every beat.
 By Ben Lacker, 2009-02-18.
 """
 import sys
-import echonest.audio as audio
+import echonest.remix.audio as audio
 
 usage = """
 Usage: 
